import os

from win32com.shell import shell, shellcon
import pkg_resources
import pythoncom


def create_microdrop_shortcut(target_path, name, description=None,
                              overwrite=False, icon=None):
    '''
    Create desktop shortcut with Microdrop version.

    Args
    ----

        batch_path (str) : Path to Microdrop launch batch file.
    '''
    # Look up Microdrop major version.
    microdrop_dist = pkg_resources.get_distribution('microdrop')
    major_version = '.'.join(microdrop_dist.version.split('.')[:2])

    # Surround target path in quotes if it contains spaces.
    if ' ' in target_path and not (target_path.startswith('"') and
                                   target_path.endswith('"')):
        target_path = '"{}"'.format(target_path)

    # Strip extension from name.
    if name.endswith('.lnk'):
        name = name[:-len('.lnk')]

    shortcut_name = name.format(target_path=target_path,
                                major_version=major_version)

    # Create shortcut instance.
    # See [here][1] for more information.
    #
    # [1]: http://docs.activestate.com/activepython/2.4/pywin32/pythoncom__CoCreateInstance_meth.html
    shortcut = pythoncom.CoCreateInstance(shell.CLSID_ShellLink, None,
                                          pythoncom.CLSCTX_INPROC_SERVER,
                                          shell.IID_IShellLink)

    # Set shortcut attributes.
    shortcut.SetPath(str(target_path))
    if description:
        shortcut.SetDescription(description.format(target_path=target_path,
                                                   major_version=major_version))
    else:
        shortcut.SetDescription(shortcut_name)

    if icon is None:
        icon_path = pkg_resources.resource_filename('microdrop', 'microdrop.ico')
    else:
        icon_path = icon

    if not (icon_path == False):
        shortcut.SetIconLocation(icon_path, 0)

    # Get `Desktop` path.
    desktop_path = shell.SHGetFolderPath(0, shellcon.CSIDL_DESKTOP, 0, 0)
    shortcut_filepath = os.path.join(desktop_path, shortcut_name + '.lnk')

    # Save shortcut to `Desktop`.
    persist_file = shortcut.QueryInterface(pythoncom.IID_IPersistFile)

    if not overwrite:
        # Get non-existing filename by appending ` (%d)` to the end of the shortcut
        # name, increasing the count until a unique name is found.
        i = 0
        while True:
            shortcut_filepath = os.path.join(desktop_path, shortcut_name +
                                             ('' if i == 0 else ' ({})'.format(i))
                                             + '.lnk')
            if not os.path.isfile(shortcut_filepath):
                break
            i += 1
    persist_file.Save(shortcut_filepath, 0)
    return shortcut_filepath



'''
Create Microdrop configuration directory.

If output directory already exists, rename directory and offer to move existing
`devices` directory (containing experiment logs).
'''
import sys

from path_helpers import path
try:
    import gtk
    import pygtkhelpers.ui.dialogs as gd
except ImportError:
    GUI_AVAILABLE = False
else:
    GUI_AVAILABLE = True

from ..config import create_config_directory


def parse_args(args=None):
    '''Parses arguments, returns (options, args).'''
    from argparse import ArgumentParser

    if args is None:
        args = sys.argv

    parser = ArgumentParser(description='Create portable MicroDrop settings '
                            'directory.')
    parser.add_argument('-g', '--gui', action='store_true', help='Use dialog '
                        'to select output directory.')
    parser.add_argument('-f', '--force-overwrite', action='store_true',
                        help='Overwrite existing files in output directory.')
    parser.add_argument('output_dir', type=path, nargs='?',
                        help='Output directory.')

    args = parser.parse_args()

    if args.gui:
        if not GUI_AVAILABLE:
            parser.error('Please install `gtk` and `pygtkhelpers`.')

        if args.output_dir is not None and not args.output_dir.isdir():
            # Starting output directory was specified, but does not exist.
            parser.error('Directory "{}" does not exist.'
                         .format(args.output_dir))
            raise SystemExit(-1)

        # Display GTK dialog to select output directory.
        folder = gd.select_folder(folder=args.output_dir, title='Select '
                                  'MicroDrop configuration output folder')
        if folder is None:
            parser.error('Folder selection cancelled.')
        folder = path(folder)
        if not args.force_overwrite and folder.isdir() and folder.listdir():
            response = gd.yesno('Output directory already exists and is not '
                                'empty.\n\nOverwrite?')
            if response == gtk.RESPONSE_YES:
                args.force_overwrite = True

        args.output_dir = folder
    elif args.output_dir is None:
        parser.error('No output directory specified.')

    return args


if __name__ == '__main__':
    args = parse_args()
    try:
        launcher_path = create_config_directory(output_dir=args.output_dir,
                                                overwrite=args.force_overwrite)
    except IOError, exception:
        print >> sys.stderr, exception
    else:
        launcher_path.parent.joinpath('devices').makedirs_p()
        launcher_path.parent.joinpath('plugins').makedirs_p()
        message = ('Start MicroDrop with the following:\n\n    {}'
                   .format('"{}"'.format(launcher_path) if ' ' in launcher_path
                           else launcher_path))
        if GUI_AVAILABLE and args.gui:
            gd.info(message)
        else:
            print message



import sys

from paver.easy import task, needs, path, sh, cmdopts, options
from paver.setuputils import setup, install_distutils_tasks
from distutils.extension import Extension
from distutils.dep_util import newer

sys.path.insert(0, path('.').abspath())
import version

setup(name='microdrop-launcher',
      version=version.getVersion(),
      description='MicroDrop launcher',
      keywords='',
      author='Christian Fobel',
      author_email='christian@fobel.net',
      url='https://github.com/wheeler-microfluidics/microdrop-launcher',
      license='GPL',
      packages=['microdrop_launcher'],
      install_requires=['appdirs', 'jinja2',
                        'microdrop-plugin-manager>=0.3.post4', 'path_helpers'],
      # Install data listed in `MANIFEST.in`
      include_package_data=True,
      entry_points = {'console_scripts':
                      ['microdrop-profile-manager = '
                       'microdrop_launcher.bin.profile_launcher:main',
                       'launch-microdrop = '
                       'microdrop_launcher.bin.launch:main']})


@task
@needs('generate_setup', 'minilib', 'setuptools.command.sdist')
def sdist():
    """Overrides sdist to make sure that our setup.py is generated."""
    pass



from datetime import datetime
import copy
import os
import pkg_resources
import sys
import tempfile
import types

from configobj import ConfigObj
import jinja2
import path_helpers as ph


# Batch file template.
launcher_template = '''
@echo off
REM Change into [parent directory of batch file][1].
REM
REM [1]: http://stackoverflow.com/questions/16623780/how-to-get-windows-batchs-parent-folder
cd "%~dp0"
REM Launch Microdrop
"{{ py_exe }}" -m microdrop.microdrop -c "%~dp0microdrop.ini"
'''

# Microdrop configuration file template.
config_template = '''
data_dir = .
[plugins]
        # directory containing microdrop plugins
        directory = plugins
[microdrop.gui.experiment_log_controller]
        notebook_directory = notebooks
[microdrop.gui.dmf_device_controller]
        device_directory = devices
'''


def directory_multi_keys(root, parent_keys=None):
    '''
    Collect all items in nested key-value mapping that satisfy one of the
    following:

     - End with ``"_dir"``.
     - Contain ``"directory"``.

    Parameters
    ----------
    root : dict-like
        Nested key-value mapping.
    parent_keys : list
        Keys of parents of ``root`` in hierarchy.

    Returns
    -------
    list
        List of tuples.

        Within each tuple, first element is a tuple of keys (one key per
        hierarchy level) and the second element is the corresponding item
        value.
    '''
    if parent_keys is None:
        parent_keys = []
    multi_keys = []
    for k, v in root.iteritems():
        if isinstance(v, dict):
            # Item is a nested dictionary.
            multi_keys_i = directory_multi_keys(v, parent_keys=
                                                parent_keys + [k])
            multi_keys.extend(multi_keys_i)
        elif isinstance(v, types.StringTypes) and any([k.endswith('_dir'),
                                                       'directory' in k]):
            # Item is a directory path
            multi_keys.append((tuple(parent_keys + [k]), ph.path(v)))
    return multi_keys


def config_relative_paths_to(config, root):
    '''
    Parameters
    ----------
    config : configobj.ConfigObj
        Configuration.
    root : str
        Root directory to resolve configuration paths against.

    Returns
    -------
    configobj.ConfigObj
        Copy of input configuration with descendent paths of ``root`` directory
        replaced with *relative* paths (with respect to ``root``).
    '''
    # Wrap `root` in `path` wrapper for convenience methods.
    root = ph.path(root)
    # Copy input config, since we are modifying it.
    config_i = copy.deepcopy(config)

    # Find config entries corresponding to directory paths.
    real_config_root = root.realpath() + os.path.sep
    dir_multi_keys = directory_multi_keys(config_i)

    # Replace descendent paths of `root` directory in config with *relative*
    # paths (with respect to `root`).
    for multi_key_i, dir_i in dir_multi_keys:
        real_dir_i = root.joinpath(dir_i).realpath()

        if os.path.commonprefix([real_dir_i + os.path.sep,
                                 real_config_root]) == real_config_root:
            # Directory is a descendent of configuration directory.

            # Replace directory reference in configuration object with
            # *relative* path to configuration directory.
            rel_path_i = root.relpathto(real_dir_i)

            config_parent_i = config_i
            for k in multi_key_i[:-1]:
                config_parent_i = config_parent_i[k]
            config_parent_i[multi_key_i[-1]] = str(rel_path_i)
    return config_i


def create_config_directory_with_paths(output_directory, paths_ini_path=None,
                                       overwrite=False):
    '''
    Create new configuration directory, where configuration file contains
    relative paths with respect to new directory.

    If :data:`paths_ini_path` is specified, replace descendent directory paths
    in :data:`paths_ini_path` with relative paths with respect to output
    directory.

    Parameters
    ----------
    output_dir : str
        Path to output directory.
    paths_ini_path : str
        Path to existing ``microdrop.ini`` file to extract directory paths
        from.
    overwrite : bool
        If ``False``, do not write to output directory if it already exists and
        is not empty.

    Returns
    -------
    path_helpers.path
        Path to launcher script (i.e., ``microdrop.bat``).
    '''
    paths_ini_path = ph.path(paths_ini_path)
    if paths_ini_path.isfile():
        # Load existing configuration file.
        paths_config = ConfigObj(paths_ini_path)
        data_dir = paths_config.get('data_dir', '.')
        paths_config_directory = (paths_ini_path.parent.joinpath(data_dir)
                                  .realpath())

        # Get configuration with descendent paths of default configuration
        # directory replaced with relative paths.
        output_config = config_relative_paths_to(config=paths_config,
                                                 root=paths_config_directory)
    else:
        output_config = None

    # Create new directory (at original path).
    launcher_path = create_config_directory(output_directory,
                                            overwrite=overwrite)

    if output_config is not None:
        output_dir_items = directory_multi_keys(output_config)

        # Open default configuration file in new directory.
        output_ini = output_directory.joinpath('microdrop.ini')
        output_config = ConfigObj(output_ini)

        # Replace directory paths in configuration with directories from
        # original configuration file.
        for multi_key_i, directory_i in output_dir_items:
            config_parent_i = output_config
            for k in multi_key_i[:-1]:
                config_parent_i = config_parent_i[k]
            config_parent_i[multi_key_i[-1]] = str(directory_i)
        # Save updated configuration file.
        output_config.write()
    return launcher_path


def create_config_directory(output_dir, overwrite=False):
    '''
    Initialize output directory with minimal Microdrop configuration.

    The created configuration causes Microdrop to store plugins and device
    files within the configuration directory.  A batch file `microdrop.bat` is
    also created in the directory to launch Microdrop using the configuration.
    Note that the ``microdrop.bat`` can be launched within any working directory.

    Parameters
    ----------
    output_dir : str
        Path to output directory.
    overwrite : bool
        If ``False``, do not write to output directory if it already exists and
        is not empty.

    Returns
    -------
    path_helpers.path
        Path to launcher script (i.e., ``microdrop.bat``).
    '''
    output_dir = ph.path(output_dir)

    if not output_dir.isdir():
        output_dir.makedirs_p()
    elif not overwrite and list(output_dir.files()):
        raise IOError('Output directory exists and is not empty.')

    config_path = output_dir.joinpath('microdrop.ini')
    with config_path.open('w') as output:
        template = jinja2.Template(config_template)
        config_str = template.render(output_dir=output_dir.name)
        output.write(config_str)

    py_exe = ph.path(sys.executable).abspath()
    launcher_path = output_dir.joinpath('microdrop.bat').abspath()
    with launcher_path.open('w') as output:
        template = jinja2.Template(launcher_template)
        launcher_str = template.render(working_dir=output_dir.abspath(),
                                       py_exe=py_exe,
                                       config_path=config_path.abspath())
        output.write(launcher_str)

    # Copy default devices to new profile.
    if pkg_resources.resource_isdir('microdrop', 'devices'):
        devices_dir = ph.path(pkg_resources.resource_filename('microdrop',
                                                              'devices'))
        devices_dir.copytree(output_dir.joinpath('devices'))

    release_version_path = output_dir.joinpath('RELEASE-VERSION')
    with release_version_path.open('w') as output:
        try:
            microdrop_dist = pkg_resources.get_distribution('microdrop')
        except Exception:
            print >> sys.stderr, ('[warning] could not find microdrop '
                                  'distribution.')
        else:
            output.write(microdrop_dist.version)

    return launcher_path


def replace_config_directory(config_directory, backup=True):
    '''
    Replace existing Microdrop configuration directory with newly initialized
    configuration.

    Directory paths from existing configuration will be copied to new
    configuration.

    Descendent paths of existing configuration directory will be replaced by
    corresponding relative paths in new configuration (with respect to new
    directory).

    Parameters
    ----------
    config_directory : str
        Path to existing Microdrop configuration directory.
    backup : True
        If ``True``, existing directory will be renamed with timestamp suffix.
        Otherwise, the existing directory will be **deleted**.

    Returns
    -------
    None or path_helpers.path
        If :data:`backup` is ``True``, the backed up directory path is
        returned.

        Otherwise, ``None`` is returned.
    '''
    config_directory = ph.path(config_directory)
    assert(config_directory.isdir())
    try:
        new_config_directory = ph.path(tempfile
                                       .mkdtemp(prefix='microdrop_config-'))

        config_ini = config_directory.joinpath('microdrop.ini')
        launcher_path = \
            create_config_directory_with_paths(new_config_directory,
                                               paths_ini_path=config_ini
                                               if config_ini.isfile()
                                               else None)

        if backup:
            # Rename existing directory with appended timestamp postfix.
            backup_name = datetime.now().strftime('{}.%Y.%m.%d-%Hh%Mm%S'
                                                  .format(config_directory.name))
            backup_directory = config_directory.parent.joinpath(backup_name)
            config_directory.rename(backup_directory)
        else:
            # **Delete original configuration directory.**
            config_directory.rmtree()

        # Rename new configuration directory with original name.
        launcher_path.parent.rename(config_directory)
    except:
        # Clean up temporary new directory.
        if new_config_directory.isdir():
            new_config_directory.rmtree()
        raise

    if backup:
        return backup_directory



import argparse
import logging
import subprocess as sp
import sys

import mpm
import mpm.bin

from ..profile import launch_profile


logger = logging.getLogger(__name__)

INSTALL_REQUIREMENTS_PARSER = \
    argparse.ArgumentParser(add_help=False,
                            parents=[mpm.bin.LOG_PARSER,
                                     mpm.bin.PLUGINS_DIR_PARSER])


def parse_args(args=None):
    '''Parses arguments, returns (options, args).'''
    if args is None:
        args = sys.argv

    parser = argparse.ArgumentParser(description='Microdrop launcher',
                                     parents=[INSTALL_REQUIREMENTS_PARSER])
    parser.add_argument('--install-plugin-requirements', action='store_true')

    return parser.parse_args()


def main(args=None):
    if args is None:
        args = parse_args()
    args = mpm.bin.validate_args(args)
    logger.debug('Arguments: %s', args)

    if args.install_plugin_requirements:
        # Run plugin "on_install" hook.
        sp.call([sys.executable, '-m', 'mpm', '-d', args.plugins_directory,
                 'hook', 'on_install'])

    if args.config_file is None:
        # No configuration file specified, so construct configuration file path
        # based on plugins directory path.
        args.config_file = (args.plugins_directory.parent
                            .joinpath('microdrop.ini'))

    return launch_profile(args.config_file.parent)


if __name__ == '__main__':
    raise SystemExit(main())



import os
import pkg_resources
import re
import subprocess as sp
import sys

import mpm
import mpm.bin
import pandas as pd
import path_helpers as ph
import yaml
try:
    import gtk
except ImportError:
    GUI_AVAILABLE = False
else:
    GUI_AVAILABLE = True

from .config import create_config_directory

cre_version = re.compile(r'^(?P<major>\d+)\.')

get_major_version = lambda version: '{}.0'.format(cre_version
                                                  .match(version)
                                                  .group('major'))

ICON_PATH = pkg_resources.resource_filename('microdrop', 'microdrop.ico')
SAVED_COLUMNS = ['used_timestamp', 'path']


def load_profiles_info(profiles_path):
    '''
    Load list of profiles from file.

    If file does not exist or list is empty, the profile list is initialized
    with the default profile directory path (creating a profile at the default
    location, if it does not already exist).

    Parameters
    ----------
    profiles_path : str
        Path to file containing list of profiles.

    Returns
    -------
    df_profiles : pandas.DataFrame
        Table of MicroDrop profile descriptions including the columns:

         - ``path`` File system path to profile directory.
         - ``used_timestamp`` Most recent time that profile was launched.
    '''
    profiles_path = ph.path(profiles_path)

    profiles_path.parent.makedirs_p()
    if profiles_path.exists():
        with profiles_path.open('r') as input_:
            profiles_str = input_.read()
            profiles = yaml.load(profiles_str)
    else:
        profiles = []

    default_profile_path = mpm.bin.get_plugins_directory().parent

    if not profiles and not default_profile_path.isdir():
        # No existing profiles.  Create default profile.
        print 'No existing profiles.  Create default profile at {}.'.format(default_profile_path)
        create_config_directory(output_dir=default_profile_path)

        for sub_directory_i in ('devices', 'plugins'):
            default_profile_path.joinpath(sub_directory_i).makedirs_p()

        # Create a `RELEASE-VERSION` file and populate it with the installed
        # MicroDrop package version.
        release_version_path = default_profile_path.joinpath('RELEASE-VERSION')
        with release_version_path.open('w') as output:
            output.write(pkg_resources.get_distribution('microdrop').version)

    if not profiles and default_profile_path.isdir():
        # No profiles list found or empty profiles list.
        #
        # Use default profile path.
        profiles = [{'path': str(default_profile_path),
                     'used_timestamp': None}]

    df_profiles = pd.DataFrame(profiles, columns=SAVED_COLUMNS)
    df_profiles.loc[df_profiles.used_timestamp == 'nan', 'used_timestamp'] = ''
    df_profiles.sort_values('used_timestamp', ascending=False, inplace=True)
    df_profiles.drop_duplicates(subset=['path'], inplace=True)
    return df_profiles


def drop_version_errors(df_profiles, missing=False, mismatch=False,
                        inplace=False):
    '''
    Drop rows for profiles with version errors.

    Parameters
    ----------
    df_profiles : pandas.DataFrame
        Table of MicroDrop profile descriptions.

        Must include *at least* the column ``path`` containing the file system
        path to each profile directory.
    missing : bool, optional
        If ``True``, drop rows for profiles where no ``RELEASE-VERSION`` file
        is found in the profile directory.
    mismatch : bool, optional
        If ``True``, drop rows for profiles where major version in
        ``RELEASE-VERSION`` file and major version of installed MicroDrop
        package **do not match**.
    inplace : bool, optional
        If ``True``, do operation inplace and return None.
    '''
    def version_error(profile_path):
        try:
            verify_profile_version(profile_path)
        except RuntimeError:
            # Major version in `RELEASE-VERSION` file and major version of
            # installed MicroDrop package **do not match**.
            return mismatch
        except IOError:
            # No `RELEASE-VERSION` file found in the profile directory.
            return missing
        else:
            return False

    error_mask = df_profiles.path.map(version_error)
    result = df_profiles.drop(error_mask[error_mask].index, inplace=inplace)
    if inplace:
        return df_profiles
    else:
        return result


def verify_profile_version(profile_path):
    '''
    Parameters
    ----------
    profile_path : str
        Path to profile directory.

    Raises
    ------
    IOError
        If no version file found in profile directory.
    RuntimeError
        If profile version does not match installed MicroDrop version.
    '''
    profile_path = ph.path(profile_path)

    release_version_path = profile_path.joinpath('RELEASE-VERSION')

    # Query the currently installed version of the MicroDrop Python package.
    installed_version_str = pkg_resources.get_distribution('microdrop').version
    installed_version = pkg_resources.parse_version(installed_version_str)

    if release_version_path.isfile():
        # A `RELEASE-VERSION` file exists in the same directory as the
        # configuration file.
        #
        # Parse the version from the `RELEASE-VERSION` file.
        release_version_str = release_version_path.lines()[0]
        release_version = pkg_resources.parse_version(release_version_str)
    else:
        # No `RELEASE-VERSION` file found in the profile directory.
        raise IOError('No version file found in profile directory.')

    if not (get_major_version(release_version_str) ==
            get_major_version(installed_version_str)):
        # Major version in `RELEASE-VERSION` file and major version of
        # installed MicroDrop package **do not match**.
        #
        # Notify the user and wait for user input to continue.
        raise RuntimeError('Configuration directory major version (%s) does '
                           'not match installed major MicroDrop version (%s)'
                           % (release_version, installed_version))


def launch_profile(profile_path):
    profile_path = ph.path(profile_path)
    config_file = profile_path.joinpath('microdrop.ini')

    try:
        verify_profile_version(profile_path)
    except IOError:
        # No `RELEASE-VERSION` file found in the profile directory.

        if GUI_AVAILABLE:
            # Prompt user to confirm profile version matches installed
            # MicroDrop version.
            dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION)
            dialog.set_icon_from_file(ICON_PATH)
            dialog.set_title('Confirm MicroDrop {} profile'
                            .format(installed_major_version()))
            dialog.add_buttons(gtk.STOCK_YES, gtk.RESPONSE_YES,
                            gtk.STOCK_NO, gtk.RESPONSE_NO)
            dialog.set_markup('Unable to determine compatible MicroDrop '
                              'version from profile:\n\n    {}\n\n'
                              'Was this profile created using the installed '
                              'version of MicroDrop ({})?'
                              .format(profile_path, installed_major_version()))
            label = (dialog.get_content_area().get_children()[0]
                     .get_children()[-1].get_children()[0])
            label.set_tooltip_text(profile_path)
            response = dialog.run()
            dialog.destroy()
            if response == gtk.RESPONSE_NO:
                raise RuntimeError('Not launching MicroDrop since profile was '
                                   'not created using the installed version of'
                                   ' MicroDrop ({})'
                                   .format(installed_major_version()))

        # Create a `RELEASE-VERSION` file and populate it with the installed
        # MicroDrop package version.
        release_version_path = profile_path.joinpath('RELEASE-VERSION')
        with release_version_path.open('w') as output:
            output.write(pkg_resources.get_distribution('microdrop').version)

    # Major version in `RELEASE-VERSION` file and major version of
    # installed MicroDrop package **match**.
    original_directory = ph.path(os.getcwd())
    try:
        # Change directory into the parent directory of the configuration
        # file.
        os.chdir(config_file.parent)
        return_code = None
        # Return code of `5` indicates program should be restarted.
        while return_code is None or return_code == 5:
            # Launch MicroDrop and save return code.
            return_code = sp.call([sys.executable, '-m',
                                    'microdrop.microdrop', '-c',
                                    config_file])
    finally:
        # Restore original working directory.
        os.chdir(original_directory)
    return return_code


def profile_major_version(profile):
    release_version_path = ph.path(profile).joinpath('RELEASE-VERSION')
    if release_version_path.isfile():
        return get_major_version(release_version_path.lines()[0])


def installed_major_version():
    installed_version_str = pkg_resources.get_distribution('microdrop').version
    return get_major_version(installed_version_str)



import datetime as dt
import functools as ft
import pkg_resources
import re
import sys

import gtk
import path_helpers as ph
import pygtkhelpers.ui.dialogs as gd
import yaml

from ..dirs import AppDirs
from ..config import create_config_directory
from ..profile import (ICON_PATH, SAVED_COLUMNS, drop_version_errors,
                       get_major_version, installed_major_version,
                       launch_profile, load_profiles_info,
                       profile_major_version)


class LaunchDialog(object):
    def __init__(self, df_profiles):
        self.df_profiles = df_profiles
        self.content_area = None
        self.frame = None
        self.profile_row = None
        self.return_code = None

    def import_profile(self, folder=None):
        # Display GTK dialog to select output directory.
        folder = gd.select_folder(folder=folder, title='Select MicroDrop '
                                  'profile directory')
        if folder is None:
            return

        major_version = profile_major_version(folder)
        self.df_profiles = self.df_profiles.append({'path': folder,
                                                    'major_version':
                                                    major_version},
                                                   ignore_index=True)
        self.df_profiles.drop_duplicates(subset=['path'], inplace=True)
        self.update_profiles_frame()

    def create_profile(self, folder=None):
        # Display GTK dialog to select output directory.
        folder = gd.select_folder(folder=folder, title='Select new MicroDrop '
                                  'profile directory')
        if folder is None:
            return
        else:
            folder = ph.path(folder).realpath()

        if folder.files() or folder.dirs():
            dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION)
            dialog.set_icon_from_file(ICON_PATH)
            dialog.set_title('Confirm overwrite')
            dialog.set_markup('Directory is not empty.\n\nOverwrite?')
            dialog.add_buttons(gtk.STOCK_YES, gtk.RESPONSE_YES,
                               gtk.STOCK_NO, gtk.RESPONSE_NO)
            response = dialog.run()
            dialog.destroy()
            if response == gtk.RESPONSE_NO:
                return
        try:
            create_config_directory(output_dir=folder, overwrite=True)
        except IOError, exception:
            print >> sys.stderr, exception
            return
        else:
            folder.joinpath('devices').makedirs_p()
            folder.joinpath('plugins').makedirs_p()

        self.df_profiles = self.df_profiles.append({'path': folder,
                                                    'major_version':
                                                    installed_major_version()},
                                                   ignore_index=True)
        self.df_profiles.drop_duplicates(subset=['path'], inplace=True)
        self.update_profiles_frame()

    def update_profiles_frame(self):
        def on_launch_clicked(profile_row_i):
            self.dialog.hide()
            self.profile_row = profile_row_i.copy()
            self.return_code = launch_profile_row(profile_row_i)
            if self.return_code is None:
                self.frame = None
                self.run()
            elif self.return_code == 0:
                profile_row_i.used_timestamp = str(dt.datetime.now())

        def on_remove_clicked(profile_row_i):
            dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION)
            dialog.set_icon_from_file(ICON_PATH)
            dialog.set_title('Remove profile')
            RESPONSE_REMOVE, RESPONSE_REMOVE_WITH_DATA, RESPONSE_CANCEL = \
                range(3)
            dialog.add_buttons('_Remove', RESPONSE_REMOVE, 'Remove with _data',
                               RESPONSE_REMOVE_WITH_DATA, 'Can_cel',
                               RESPONSE_CANCEL)
            dialog.set_markup('Remove the following profile from list?\n\n'
                              '    {}\n\n'
                              '<b>"Remove with data"</b> removes profile from '
                              'list <b>and deletes the profile '
                              'directory</b>.'.format(profile_row_i.path))
            response = dialog.run()
            dialog.destroy()
            if response not in (RESPONSE_REMOVE, RESPONSE_REMOVE_WITH_DATA):
                return
            try:
                if response == RESPONSE_REMOVE_WITH_DATA:
                    dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION)
                    dialog.set_icon_from_file(ICON_PATH)
                    dialog.set_title('Confirm profile delete')
                    dialog.set_markup('Remove profile data (cannot be '
                                      'undone)?')
                    dialog.add_buttons(gtk.STOCK_YES, gtk.RESPONSE_YES,
                                       gtk.STOCK_NO, gtk.RESPONSE_NO)
                    response = dialog.run()
                    dialog.destroy()
                    if response == gtk.RESPONSE_YES:
                        ph.path(profile_row_i.path).rmtree()
                    else:
                        return
            except Exception, exception:
                gd.error(str(exception))
            finally:
                self.df_profiles = (self.df_profiles
                                    .loc[self.df_profiles.path !=
                                         profile_row_i.path].copy())
            self.update_profiles_frame()

        if self.frame is not None:
            self.content_area.remove(self.frame)
        self.frame = get_profiles_table(self.df_profiles, on_launch_clicked,
                                        on_remove_clicked)
        self.content_area.pack_start(self.frame, expand=True, fill=True, padding=10)
        self.content_area.reorder_child(self.frame, 0)

    def run(self):
        self.dialog = gtk.Dialog()
        self.dialog.set_icon_from_file(ICON_PATH)
        self.dialog.set_title('MicroDrop Profile Manager')
        self.content_area = self.dialog.get_content_area()

        buttons_area = gtk.HBox()
        buttons_box = gtk.HButtonBox()
        button_import = gtk.Button('Import...')
        button_create = gtk.Button('Create...')
        button_import.connect('clicked', lambda *args:
                              self.import_profile(self.df_profiles
                                                  .path.get(0)))
        button_create.connect('clicked', lambda *args:
                              self.create_profile(self.df_profiles.path
                                                  .get(0)))
        for button_i in (button_import, button_create):
            buttons_box.pack_end(button_i, expand=False, fill=False)
        buttons_area.pack_end(buttons_box, expand=False, fill=False)
        buttons_area.show_all()
        self.content_area.pack_end(buttons_area, expand=False, fill=False)

        self.update_profiles_frame()
        self.dialog.show()
        table = self.frame.get_child().get_child().get_child()
        x, y, width, height = table.get_allocation()
        self.dialog.set_size_request(width + 30, 320)
        self.dialog.props.resizable = False
        self.dialog.run()

        return self.profile_row


def get_profiles_table(df_profiles, launch_callback, remove_callback,
                       short_threshold=40):
    def short_path(path_i):
        base_i, name_i = ph.path(path_i).splitpath()

        if len(base_i) > short_threshold:
            base_i = ph.path(base_i[:short_threshold]).parent.joinpath('...')
        return base_i.joinpath(name_i)

    grid_columns = ['path', 'major_version']
    # One header row plus one row per profile
    # One column for each column in `grid_columns`, plus two columns for launch
    # and remove buttons, respectively.
    table = gtk.Table(df_profiles.shape[0] + 1, len(grid_columns) + 2)

    for i, column_i in enumerate(grid_columns):
        label_i = gtk.Label()
        label_i.set_markup('<b>{}</b>'.format(column_i))
        table.attach(label_i, left_attach=i, right_attach=i + 1, top_attach=0,
                     bottom_attach=1, xpadding=5, ypadding=5,
                     xoptions=gtk.SHRINK, yoptions=gtk.SHRINK)

    for i, (ix, row_i) in enumerate(df_profiles.iterrows()):
        row_kwargs = dict(top_attach=i + 1, bottom_attach=i + 2,
                          xpadding=5, ypadding=5,
                          xoptions=gtk.SHRINK | gtk.FILL, yoptions=gtk.SHRINK)
        for j, column_ij in enumerate(grid_columns):
            if column_ij == 'path':
                label_ij = gtk.Label(short_path(row_i.path))
                timestamp_str_ij = re.sub(r'\.\d+$', '',
                                          str(row_i.used_timestamp))
                label_ij.set_tooltip_text('{}\nLast used: {}'
                                          .format(row_i.path,
                                                  timestamp_str_ij))
                label_ij.set_alignment(0, .5)
            else:
                label_ij = gtk.Label(row_i[column_ij])
            table.attach(label_ij, left_attach=j, right_attach=j + 1,
                         **row_kwargs)

        button_launch_i = gtk.Button('Launch')
        button_open_i = gtk.Button('Open')
        button_remove_i = gtk.Button('Remove')
        on_launch_clicked = ft.partial(launch_callback, row_i)
        on_open_clicked = ft.partial(ph.path(row_i.path).launch)
        on_remove_clicked = ft.partial(remove_callback, row_i)
        button_launch_i.connect('clicked', lambda *args: on_launch_clicked())
        button_open_i.connect('clicked', lambda *args: on_open_clicked())
        button_remove_i.connect('clicked', lambda *args: on_remove_clicked())
        # button_remove_i.connect('clicked', lambda
        for button_ij, j in zip((button_launch_i, button_open_i,
                                 button_remove_i), range(j + 1, j + 4)):
            table.attach(button_ij, left_attach=j, right_attach=j + 1,
                         **row_kwargs)

    scrolled_window = gtk.ScrolledWindow()
    scrolled_window.set_policy(hscrollbar_policy=gtk.POLICY_AUTOMATIC,
                               vscrollbar_policy=gtk.POLICY_ALWAYS)
    scrolled_window.add_with_viewport(table)
    scrolled_window.props.shadow_type = gtk.SHADOW_NONE
    scrolled_window.get_child().props.shadow_type = gtk.SHADOW_NONE
    frame = gtk.Frame(label='Select profile to launch')
    frame.add(scrolled_window)
    # frame.props.shadow_type = gtk.SHADOW_NONE
    frame.show_all()
    return frame


def launch_profile_row(profile_row_i):
    try:
        return_code = launch_profile(profile_row_i.path)
    except Exception, exception:
        dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
                                   message_format=str(exception))
        dialog.set_icon_from_file(ICON_PATH)
        dialog.set_title('Error launching profile')
        dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK)
        dialog.run()
        dialog.destroy()
    else:
        return return_code


def parse_args(args=None):
    '''Parses arguments, returns (options, args).'''
    from argparse import ArgumentParser

    if args is None:
        args = sys.argv

    major_version = get_major_version(pkg_resources
                                      .get_distribution('microdrop').version)
    # Look up MicroDrop application directories based on major version.
    microdrop_env_dirs = AppDirs('MicroDrop', version=major_version)
    # Construct path to list of profiles based on user configuration directory.
    default_profiles_path = (microdrop_env_dirs.user_config_dir
                             .joinpath('profiles.yml'))

    parser = ArgumentParser(description='MicroDrop {} profile manager'
                            .format(major_version))

    parser.add_argument('-f', '--profiles-path', type=ph.path,
                        help='Path to profiles list (default=%(default)s)',
                        default=default_profiles_path)
    parser.add_argument('--default', action='store_true',
                        help='Launch most recently used profile.')
    parser.add_argument('--no-auto', action='store_true',
                        help='If not set and there is only a single profile, '
                        'MicroDrop is launched using the profile.')

    args = parser.parse_args()

    return args


def main():
    args = parse_args()

    # Load list of profiles from file.
    #
    # If file does not exist or list is empty, the profile list is initialized
    # with the default profile directory path.
    df_profiles = load_profiles_info(args.profiles_path)
    drop_version_errors(df_profiles, missing=False, mismatch=True,
                        inplace=True)

    # Save most recent list of profiles to disk.
    with args.profiles_path.open('w') as output:
        profiles_str = yaml.dump(df_profiles[SAVED_COLUMNS].astype(str)
                                 .to_dict('records'), default_flow_style=False)
        output.write(profiles_str)

    # Look up major version of each profile.
    df_profiles['major_version'] = df_profiles.path.map(profile_major_version)

    if args.default or (not args.no_auto and df_profiles.shape[0] == 1):
        # Launch MicroDrop with most recently used (or only available) profile.
        return_code = launch_profile_row(df_profiles.iloc[0])
        if return_code == 0:
            df_profiles.used_timestamp[0] = str(dt.datetime.now())
    else:
        # Display dialog to manage profiles or launch a profile.
        launch_dialog = LaunchDialog(df_profiles)
        launch_dialog.run()
        return_code = launch_dialog.return_code
        df_profiles = launch_dialog.df_profiles

    # Save most recent list of profiles to disk (most recently used first).
    #
    # List can be changed using dialog by:
    #  - Creating a new profile.
    #  - Importing a profile.
    #  - Updating used timestamp by launching a profile.
    df_profiles = df_profiles.astype(str)
    df_profiles.loc[df_profiles.used_timestamp == 'nan', 'used_timestamp'] = ''
    df_profiles.sort_values('used_timestamp', ascending=False, inplace=True)

    with args.profiles_path.open('w') as output:
        profiles_str = yaml.dump(df_profiles[SAVED_COLUMNS]
                                 .to_dict('records'), default_flow_style=False)
        output.write(profiles_str)

    return return_code


if __name__ == '__main__':
    return_code = main()
    raise SystemExit(return_code)



import os
import sys

import appdirs
import path_helpers as ph


class AppDirs(appdirs.AppDirs):
    """Convenience wrapper for getting application dirs."""
    def __init__(self, *args, **kwargs):
        if any(['continuum analytics, inc.' in sys.version.lower(),
                'conda' in sys.version.lower()]):
            # Assume running under Conda.
            if 'CONDA_PREFIX' in os.environ:
                self.conda_prefix = ph.path(os.environ['CONDA_PREFIX'])
            else:
                # Infer Conda prefix as parent directory of Python executable.
                self.conda_prefix = ph.path(sys.executable).parent.realpath()
        else:
            # Assume running under Conda.
            self.conda_prefix = None
        super(AppDirs, self).__init__(*args, **kwargs)

    def _get_conda_path(self, base):
        sub_directories = map(str, [v for v in (self.appauthor, self.appname,
                                                self.version) if v])
        return self.conda_prefix.joinpath(*[base] + sub_directories)

    @property
    def user_data_dir(self):
        if self.conda_prefix is None:
            return ph.path(appdirs.AppDirs.user_data_dir.fget(self))
        else:
            return self._get_conda_path('share')

    @property
    def site_data_dir(self):
        if self.conda_prefix is None:
            return ph.path(appdirs.AppDirs.site_data_dir.fget(self))
        else:
            return self.user_data_dir

    @property
    def user_config_dir(self):
        if self.conda_prefix is None:
            return ph.path(appdirs.AppDirs.user_config_dir.fget(self))
        else:
            return self._get_conda_path('etc')

    @property
    def site_config_dir(self):
        if self.conda_prefix is None:
            return ph.path(appdirs.AppDirs.site_config_dir.fget(self))
        else:
            return self.user_config_dir

    @property
    def user_cache_dir(self):
        if self.conda_prefix is None:
            return ph.path(appdirs.AppDirs.user_cache_dir.fget(self))
        else:
            return self._get_conda_path('cache')

    @property
    def user_log_dir(self):
        if self.conda_prefix is None:
            return ph.path(appdirs.AppDirs.user_log_dir.fget(self))
        else:
            return self._get_conda_path('cache').joinpath('log')
